#!/usr/bin/env chibi-scheme
;;; getopt-test.scm --- tests for (nytpu getopt)
;;;
;;; Copyright (c) 2021 nytpu <alex [at] nytpu.com>
;;; SPDX-License-Identifier: MPL-2.0
;;; Home Page: <https://git.sr.ht/~nytpu/getopt-scheme>

(import (scheme base)
        (srfi 64)
        (nytpu getopt))

(test-begin "getopt")
(test-begin "valid calls")
(let ((result (getopt "abcd" "getopt" "-ab" "-c" "--" "-d" "argument")))
  (test-assert (assoc 'command result))
  (test-equal "getopt" (cdr (assoc 'command result)))

  (test-assert (assoc #\a result))
  (test-assert (assoc #\b result))
  (test-assert (assoc #\c result))
  (test-assert (not (assoc #\d result)))

  (test-assert (assoc '@ result))
  (test-equal '("-d" "argument") (cdr (assoc '@ result))))

(let ((result (getopt ":abcd" "getopt-2" "-abc" "argument" "-d")))
  (test-assert (assoc 'command result))
  (test-equal "getopt-2" (cdr (assoc 'command result)))

  (test-assert (assoc #\a result))
  (test-assert (assoc #\b result))
  (test-assert (assoc #\c result))
  (test-assert (not (assoc #\d result)))

  (test-assert (assoc '@ result))
  (test-equal '("argument" "-d") (cdr (assoc '@ result))))

(let ((result (getopt "a:b:c:d:" "getopt" "-abc" "-bhello" "-d" "test")))
  (test-assert (assoc 'command result))
  (test-equal "getopt" (cdr (assoc 'command result)))

  (test-assert (assoc #\a result))
  (test-equal "bc" (cdr (assoc #\a result)))

  (test-assert (assoc #\b result))
  (test-equal "hello" (cdr (assoc #\b result)))

  (test-assert (not (assoc #\c result)))

  (test-assert (assoc #\d result))
  (test-equal "test" (cdr (assoc #\d result)))

  (test-assert (not (assoc '@ result))))

(let ((result (getopt ":a:b:c:d:" "getopt" "-abc" "-bhello" "-d" "test" "arg")))
  (test-assert (assoc 'command result))
  (test-equal "getopt" (cdr (assoc 'command result)))

  (test-assert (assoc #\a result))
  (test-equal "bc" (cdr (assoc #\a result)))

  (test-assert (assoc #\b result))
  (test-equal "hello" (cdr (assoc #\b result)))

  (test-assert (not (assoc #\c result)))

  (test-assert (assoc #\d result))
  (test-equal "test" (cdr (assoc #\d result)))

  (test-assert (assoc '@ result))
  (test-equal '("arg") (cdr (assoc '@ result))))

(let ((result (getopt "ab:cd" "getopt" "argument")))
  (test-assert (assoc 'command result))
  (test-equal "getopt" (cdr (assoc 'command result)))

  (test-assert (not (assoc #\a result)))
  (test-assert (not (assoc #\b result)))
  (test-assert (not (assoc #\c result)))
  (test-assert (not (assoc #\d result)))

  (test-assert (assoc '@ result))
  (test-equal '("argument") (cdr (assoc '@ result))))

(let ((result (getopt ":ab:cd" "getopt")))
  (test-assert (assoc 'command result))
  (test-equal "getopt" (cdr (assoc 'command result)))

  (test-assert (not (assoc #\a result)))
  (test-assert (not (assoc #\b result)))
  (test-assert (not (assoc #\c result)))
  (test-assert (not (assoc #\d result)))

  (test-assert (not (assoc '@ result))))
(test-end "valid calls")

(test-begin "errors")
(test-error (getopt "a:bcd" "getopt" "-a"))
(test-error (getopt "a:bcd" "getopt" "-bcda"))
(test-error (getopt ":abcd" "getopt" "-abcdg"))
(test-end "errors")
(test-end "getopt")
